#
# ZBabelTag
# (C) Copyright 2000 The Internet (Aust) Pty Ltd
# ACN: 082 081 472  ABN: 83 082 081 472
#
# Author: Andrew Milton <akm@theinternet.com.au>
#
# USAGE:
#
# <dtml-babel src="language" dst="language">
# TheText
# </dtml-babel>
#
# src: is the language that TheText written in
# dst: is the desired language that TheText is to be displayed in.
#
# You can use any designations you like for language.

from Globals import Persistent, HTMLFile, HTML, MessageDialog
from AccessControl.Role import RoleManager
import Acquisition, sys, string, types, mimetools
import OFS.SimpleItem
import Globals
from cStringIO import StringIO
import md5, base64

manage_addZBabelTowerForm=HTMLFile('manageAdd.dtml',globals())

def manage_addZBabelTower(self, id='ZBabelTower', REQUEST=None):
    """ Add a Babel Tower """

    ob = ZBabelTower(id)
    self._setObject(od,ob)
    root = getattr(self,id)

    # Add the TowerOfBabel from which we can find stuff.
    root._setObject('TowerOfBabel',ZCatalog('TowerOfBabel'))
    cat = getattr(root,'TowerOfBabel')

    cat._catalog.addColumn('md5Hash')
    cat._catalog.addColumn('renderText')
    # This will probably be redundant
    # But it does give you where the source translation
    # came from.
    cat._catalog.addColumn('srcLang')
    cat._catalog.addColumn('dstLang')
    cat._catalog.addColumn('textPath')

    cat._catalog.addColumn('language')

    cat._catalog.addIndex('md5Hash','FieldIndex')
    cat._catalog.addIndex('srcLang','FieldIndex')
    cat._catalog.addIndex('dstLang','FieldIndex')
    cat._catalog.addIndex('textPath','FieldIndex')

    root._catalog = cat._catalog

    if REQUEST:
        return self.manage_main(self,REQUEST)

class ZBabelTower(Acquisition.Implicit,
                 OFS.SimpleItem,
                 RoleManager):
    ''' A Translation System '''

    meta_type='ZBabel'

    manage_options =  (
        {'label':'Edit','action':'manage_main'},
        {'label':'Translations', 'action':'manage_Translate'},
        {'label':'Security','action':'manage_access'},
        )

    __ac_permission__ = (
        ('Manage ZBabel Towers', ('manage_main', 'manage',
                                  'manage_workspace',
                                  'manage_Translate',
                                  'reCatalog')),
        )

    manage = HTMLFile('manage.dtml',globals())
    manage_main = manage_workspace = manage

    manage_Translate = HTMLFile('Translations.dtml',globals())
    
    def __init__(self,id):
        self.id=id

    def findTranslations(self, digest, srcText, src, dst,srcPath):
        ourCatalog={}
        if hasattr(self,'TowerOfBabel'):
            ourCatalog=self.getattr('TowerOfBable')
        else:
            return None
        
        matches=ourCatalog.searchResults(id=digest)
        if matches:
            for hit in matches:
                if hit['dstLang'] == dst:
                    return hit['renderText']
            # No native return the same text
            return srcText
        else:
            # Nothing registered
            # Add it to the catalog
            newTrans={}
            newTrans['md5Hash']=digest
            newTrans['srcLang']=src
            newTrans['dstLang']=dst
            newTrans['renderText']=srcText
            newTrans['renderPath']=srcPath
            ourCatalog.catalog_object(ourCatalog,newTrans,digest)
        
    def translate(self, srcText, src, dst, srcPath):
        digest=md5.new()
        digest.update(srcText)
        digest=digest.digest()
        digest=string.strip(base64.encodestring(digest))
        matchText=self.findTranslation(digest,srcText,src,dst,srcPath)
        return matchText
        
